@@ -1,5 +1,5 @@
 /* gtktools.c  -  The GNU Privacy Assistant
- *	Copyright (C) 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 2000, 2001 G-N-U GmbH.
  *
  * This file is part of GPA
  *
@@ -24,28 +24,37 @@
 #include <gtk/gtk.h>
 #include "gpa.h"
 #include "gpawindowkeeper.h"
+#include "icons.h"
 
-void
-gpa_widget_set_centered (GtkWidget * widget, GtkWidget * parent)
-{
-/* var */
-  int x0, y0, w0, h0, x, y, w, h;
-/* commands */
-  gdk_window_get_size (widget->window, &w, &h);
-  gdk_window_get_origin (parent->window, &x0, &y0);
-  gdk_window_get_size (parent->window, &w0, &h0);
-  x = x0 + (w0 - w) / 2;
-  y = y0 + (h0 - h) / 2;
-  gdk_window_move_resize (widget->window, x, y, w, h);
-}				/* gpa_widget_set_centered */
 
+/* Show all children of the GtkWindow widget, realize it, set its
+ * position so that it's in the center of the parent and show the window
+ * itself.
+ */
 void
-gpa_widget_show (GtkWidget * widget, GtkWidget * parent, gchar * tip)
+gpa_window_show_centered (GtkWidget * widget, GtkWidget * parent)
 {
-    gtk_widget_show_all (widget);
-    gpa_widget_set_centered (widget, parent);
-    gpa_windowTip_show (tip);
-}				/* gpa_widget_show */
+  int parent_x, parent_y, parent_width, parent_height;
+  int center_x, center_y;
+  int width, height;
+  GtkWidget * child = GTK_BIN (widget)->child;
+
+  gtk_widget_show_all (child);
+  gtk_widget_realize (child);
+
+  gdk_window_get_size (widget->window, &width, &height);
+  gdk_window_get_origin (parent->window, &parent_x, &parent_y);
+  gdk_window_get_size (parent->window, &parent_width, &parent_height);
+
+  center_x = parent_x + (parent_width - width) / 2;
+  center_y = parent_y + (parent_height - height) / 2;
+
+  gtk_window_set_position (GTK_WINDOW(widget), GTK_WIN_POS_NONE);
+  gtk_widget_set_uposition (widget, center_x, center_y);
+  gtk_window_set_transient_for(GTK_WINDOW(widget), GTK_WINDOW(parent));
+  gtk_widget_show_all (widget);
+} /* gpa_window_show_centered */
+
 
 void
 gpa_window_destroy (gpointer param)
@@ -53,25 +62,17 @@ gpa_window_destroy (gpointer param)
 /* var */
   gpointer *localParam;
   GpaWindowKeeper *keeper;
-  gchar *tip;
+
 /* commands */
   localParam = (gpointer *) param;
   keeper = (GpaWindowKeeper *) localParam[0];
-  tip = (gchar *) localParam[1];
   gpa_windowKeeper_release (keeper);
-  if (tip)
-    {
-      if (strcmp (tip, "") != 0)
-	gpa_windowTip_show (tip);
-    }				/* if */
-  else
-    gtk_widget_hide (global_windowTip);
 }				/* gpa_window_destroy */
 
 GtkWidget *
 gpa_space_new (void)
 {
-  return gtk_label_new (_(""));
+  return gtk_label_new ("");
 }				/* gpa_space_new */
 
 GtkWidget *
@@ -109,7 +110,7 @@ gpa_button_new (GtkAccelGroup * accelGroup, gchar * labelText)
 /* objects */
   GtkWidget *button;
 /* commands */
-  button = gtk_button_new_with_label (_(""));
+  button = gtk_button_new_with_label ("");
   accelKey =
     gtk_label_parse_uline (GTK_LABEL (GTK_BIN (button)->child), labelText);
   gtk_widget_add_accelerator (button, "clicked", accelGroup, accelKey,
@@ -119,11 +120,12 @@ gpa_button_new (GtkAccelGroup * accelGroup, gchar * labelText)
 
 /* Create a new hbox with an image and a label packed into it
  * and return the box. Then treat the accelerators. */
-GtkWidget *gpa_xpm_label_box( GtkWidget *parent,
-						 gchar	   **xpm,
-						 gchar	   *label_text,
-						 GtkWidget * button,
-						 GtkAccelGroup * accelGroup)
+GtkWidget *
+gpa_xpm_label_box( GtkWidget *parent,
+                   const char *xpm_name,
+                   gchar	   *label_text,
+                   GtkWidget * button,
+                   GtkAccelGroup * accelGroup)
 {
    guint accelKey;
    GtkWidget *box1;
@@ -131,20 +133,13 @@ GtkWidget *gpa_xpm_label_box( GtkWidget *parent,
    GtkWidget *pixmapwid;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
-   GtkStyle *style;
 
    /* Create box for xpm and label */
    box1 = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box1), 2);
 
-   /* Get the style of the button to get the
-	* background color. */
-   style = gtk_widget_get_style(parent);
-
    /* Now on to the xpm stuff */
-   pixmap = gdk_pixmap_create_from_xpm_d (parent->window, &mask,
-						&style->bg[GTK_STATE_NORMAL],
-						(gchar **)xpm);
+   pixmap = gpa_create_icon_pixmap ( parent, xpm_name, &mask );
    pixmapwid = gtk_pixmap_new (pixmap, mask);
 
    /* Create a label for the button */
@@ -155,16 +150,16 @@ GtkWidget *gpa_xpm_label_box( GtkWidget *parent,
   gtk_widget_add_accelerator (button, "clicked", accelGroup, accelKey,
 			      GDK_MOD1_MASK, 0);
 
-   /* Pack the pixmap and label into the box */
-   gtk_box_pack_start (GTK_BOX (box1),
-			   pixmapwid, FALSE, FALSE, 3);
+  /* Pack the pixmap and label into the box */
+  gtk_box_pack_start (GTK_BOX (box1),
+                      pixmapwid, FALSE, FALSE, 3);
+  
+  gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);
+  
+  gtk_widget_show(pixmapwid);
+  gtk_widget_show(label);
 
-   gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);
-
-   gtk_widget_show(pixmapwid);
-   gtk_widget_show(label);
-
-   return(box1);
+  return(box1);
 }
 
 GtkWidget *
@@ -183,6 +178,20 @@ gpa_buttonCancel_new (GtkAccelGroup * accelGroup, gchar * labelText,
   return (buttonCancel);
 }				/* gpa_buttonCancel_new */
 
+/* an extended version of gpa_buttonCancel_new */
+GtkWidget *
+gpa_button_cancel_new (GtkAccelGroup * accel, gchar * label,
+		       GtkSignalFunc func, gpointer param)
+{
+  GtkWidget *button;
+
+  button = gpa_button_new (accel, label);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked", func, param);
+  gtk_widget_add_accelerator (button, "clicked", accel, GDK_Escape, 0, 0);
+  return button;
+}				/* gpa_button_cancel_new */
+
+
 GtkWidget *
 gpa_check_button_new (GtkAccelGroup * accelGroup, gchar * labelText)
 {
@@ -191,7 +200,7 @@ gpa_check_button_new (GtkAccelGroup * accelGroup, gchar * labelText)
 /* objects */
   GtkWidget *checker;
 /* commands */
-  checker = gtk_check_button_new_with_label (_(""));
+  checker = gtk_check_button_new_with_label ("");
   accelKey =
     gtk_label_parse_uline (GTK_LABEL (GTK_BIN (checker)->child), labelText);
   gtk_widget_add_accelerator (checker, "clicked", accelGroup, accelKey,
@@ -207,7 +216,7 @@ gpa_radio_button_new (GtkAccelGroup * accelGroup, gchar * labelText)
 /* objects */
   GtkWidget *radio;
 /* commands */
-  radio = gtk_radio_button_new_with_label (NULL, _(""));
+  radio = gtk_radio_button_new_with_label (NULL, "");
   accelKey =
     gtk_label_parse_uline (GTK_LABEL (GTK_BIN (radio)->child), labelText);
   gtk_widget_add_accelerator (radio, "clicked", accelGroup, accelKey,
@@ -225,7 +234,7 @@ gpa_radio_button_new_from_widget (GtkRadioButton * widget,
 /* objects */
   GtkWidget *radio;
 /* commands */
-  radio = gtk_radio_button_new_with_label_from_widget (widget, _(""));
+  radio = gtk_radio_button_new_with_label_from_widget (widget, "");
   accelKey =
     gtk_label_parse_uline (GTK_LABEL (GTK_BIN (radio)->child), labelText);
   gtk_widget_add_accelerator (radio, "clicked", accelGroup, accelKey,
@@ -241,7 +250,7 @@ gpa_toggle_button_new (GtkAccelGroup * accelGroup, gchar * labelText)
 /* objects */
   GtkWidget *toggle;
 /* commands */
-  toggle = gtk_toggle_button_new_with_label (_(""));
+  toggle = gtk_toggle_button_new_with_label ("");
   accelKey =
     gtk_label_parse_uline (GTK_LABEL (GTK_BIN (toggle)->child), labelText);
   gtk_widget_add_accelerator (toggle, "clicked", accelGroup, accelKey,
@@ -261,22 +270,47 @@ gpa_connect_by_accelerator (GtkLabel * label, GtkWidget * widget,
 			      GDK_MOD1_MASK, 0);
 }				/* gpa_connect_by_accelerator */
 
+
+/* Set the text of the GtkButton button to text, remove existing
+ * accelerators and and add new accelerators if accel_group is not NULL
+ */
+void
+gpa_button_set_text (GtkWidget * button, gchar * text,
+		     GtkAccelGroup * accel_group)
+{
+  GtkWidget * label = GTK_BIN (button)->child;
+  guint accel;
+
+  accel = gtk_label_parse_uline (GTK_LABEL (label), text);
+
+  /* In GTK 1.2.8, the visible_only parameter of
+   * gtk_widget_remove_accelerators is not even looked at, it always
+   * behaves as if it were TRUE. Therefore make sure that
+   * gtk_widget_add_accelerator is called with the GTK_ACCEL_VISIBLE
+   * flag set to make remvoing the accels work.
+   */
+  gtk_widget_remove_accelerators (button, "clicked", TRUE);
+  if (accel_group && accel != GDK_VoidSymbol)
+    {
+      gtk_widget_add_accelerator (button, "clicked", accel_group,
+				  accel, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
+    }
+}
+
 void
 gpa_window_error (gchar * message, GtkWidget * messenger)
 {
-/* var */
   GpaWindowKeeper *keeper;
   GtkAccelGroup *accelGroup;
-/* objects */
   GtkWidget *windowError;
   GtkWidget *vboxError;
   GtkWidget *labelMessage;
   GtkWidget *buttonClose;
-/* commands */
+
   keeper = gpa_windowKeeper_new ();
   windowError = gtk_window_new (GTK_WINDOW_DIALOG);
   gpa_windowKeeper_set_window (keeper, windowError);
-  gtk_window_set_title (GTK_WINDOW (windowError), "GPA Error");
+  gtk_window_set_title (GTK_WINDOW (windowError), _("GPA Error"));
   gtk_window_set_modal (GTK_WINDOW (windowError), TRUE);
   accelGroup = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (windowError), accelGroup);
@@ -292,10 +326,13 @@ gpa_window_error (gchar * message, GtkWidget * messenger)
 			      0, 0);
   gtk_box_pack_start (GTK_BOX (vboxError), buttonClose, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (windowError), vboxError);
-  gtk_widget_show_all (windowError);
-  gpa_widget_set_centered (windowError, messenger);
+
+  if (messenger)
+    gpa_window_show_centered (windowError, messenger);
+  else
+    gtk_widget_show_all (windowError);
   gtk_widget_grab_focus (buttonClose);
-}				/* gpa_window_error */
+} /* gpa_window_error */
 
 void
 gpa_window_message (gchar * message, GtkWidget * messenger)
@@ -312,7 +349,7 @@ gpa_window_message (gchar * message, GtkWidget * messenger)
   keeper = gpa_windowKeeper_new ();
   windowMessage = gtk_window_new (GTK_WINDOW_DIALOG);
   gpa_windowKeeper_set_window (keeper, windowMessage);
-  gtk_window_set_title (GTK_WINDOW (windowMessage), "GPA Message");
+  gtk_window_set_title (GTK_WINDOW (windowMessage), _("GPA Message"));
   gtk_window_set_modal (GTK_WINDOW (windowMessage), TRUE);
   accelGroup = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (windowMessage), accelGroup);
@@ -328,11 +365,96 @@ gpa_window_message (gchar * message, GtkWidget * messenger)
 			      0, 0);
   gtk_box_pack_start (GTK_BOX (vboxMessage), buttonClose, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (windowMessage), vboxMessage);
-  gtk_widget_show_all (windowMessage);
-  gpa_widget_set_centered (windowMessage, messenger);
+  gpa_window_show_centered (windowMessage, messenger);
   gtk_widget_grab_focus (buttonClose);
 }				/* gpa_window_message */
 
+
+/* a simple but flexible message box */
+
+
+typedef struct {
+  gchar * result;
+  GtkWidget * window;
+} GPAMessageBox;
+
+static void
+message_box_destroy (GtkWidget *widget, gpointer param)
+{
+  gtk_main_quit ();
+}
+
+static void
+message_box_clicked (GtkWidget * button, gpointer param)
+{
+  GPAMessageBox * dialog = param;
+
+  dialog->result = gtk_object_get_data (GTK_OBJECT (button), "user_data");
+  gtk_widget_destroy (dialog->window);
+}
+
+gchar *
+gpa_message_box_run (GtkWidget * parent, const gchar * title,
+		     const gchar * message,
+		     const gchar ** buttons)
+{
+  GtkAccelGroup *accel_group;
+
+  GtkWidget * window;
+  GtkWidget * vbox;
+  GtkWidget * label;
+  GtkWidget * bbox;
+  GtkWidget * button;
+  int i;
+  GPAMessageBox dialog;
+  
+
+  dialog.result = NULL;
+
+  window = gtk_window_new (GTK_WINDOW_DIALOG);
+  dialog.window = window;
+  gtk_window_set_title (GTK_WINDOW (window), title);
+  gtk_signal_connect (GTK_OBJECT (window), "destroy",
+		      GTK_SIGNAL_FUNC (message_box_destroy),
+		      NULL);
+
+  accel_group = gtk_accel_group_new ();
+  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+
+  vbox = gtk_vbox_new (FALSE, 5);
+  gtk_container_add (GTK_CONTAINER (window), vbox);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+
+  label = gtk_label_new (message);
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+
+  bbox = gtk_hbutton_box_new ();
+  gtk_box_pack_start (GTK_BOX (vbox), bbox, TRUE, TRUE, 0);
+  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
+  gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox), 10);
+
+  for (i = 0; buttons[i]; i++)
+    {
+      button = gpa_button_new (accel_group, (gchar*)(buttons[i]));
+      gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+      gtk_object_set_data (GTK_OBJECT (button), "user_data",
+			   (gpointer)buttons[i]);
+      gtk_signal_connect (GTK_OBJECT (button), "clicked",
+			  GTK_SIGNAL_FUNC (message_box_clicked),
+			  (gpointer)&dialog);
+    }
+
+  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+  gpa_window_show_centered (window, parent);
+
+  gtk_main ();
+
+  return dialog.result;
+}
+  
+
 void
 gpa_window_passphrase (GtkWidget * messenger, GtkSignalFunc func, gchar * tip,
 		       gpointer data)
@@ -363,7 +485,7 @@ gpa_window_passphrase (GtkWidget * messenger, GtkSignalFunc func, gchar * tip,
   gtk_container_set_border_width (GTK_CONTAINER (vboxPassphrase), 5);
   hboxPasswd = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (hboxPasswd), 5);
-  labelPasswd = gtk_label_new (_(""));
+  labelPasswd = gtk_label_new ("");
   gtk_box_pack_start (GTK_BOX (hboxPasswd), labelPasswd, FALSE, FALSE, 0);
   entryPasswd = gtk_entry_new ();
   gtk_entry_set_visibility (GTK_ENTRY (entryPasswd), FALSE);
@@ -396,7 +518,81 @@ gpa_window_passphrase (GtkWidget * messenger, GtkSignalFunc func, gchar * tip,
   gtk_box_pack_start (GTK_BOX (vboxPassphrase), hButtonBoxPassphrase, FALSE,
 		      FALSE, 0);
   gtk_container_add (GTK_CONTAINER (windowPassphrase), vboxPassphrase);
-  gtk_widget_show_all (windowPassphrase);
-  gpa_widget_set_centered (windowPassphrase, messenger);
+  gpa_window_show_centered (windowPassphrase, messenger);
   gtk_widget_grab_focus (entryPasswd);
 }				/* gpa_window_passphrase */
+
+/*
+ * Modal file dialog
+ */
+
+struct _GPASaveFileNameDialog {
+  GtkWidget * window;
+  gchar * filename;
+};
+typedef struct _GPASaveFileNameDialog GPASaveFileNameDialog;
+
+static void
+file_dialog_ok (gpointer param)
+{
+  GPASaveFileNameDialog * dialog = param;
+
+  dialog->filename \
+      = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog->window));
+  dialog->filename = xstrdup (dialog->filename);
+
+  gtk_widget_destroy (dialog->window);
+}
+
+static void
+file_dialog_cancel (gpointer param)
+{
+  GPASaveFileNameDialog * dialog = param;
+  dialog->filename = NULL;
+  gtk_widget_destroy (dialog->window);
+}
+
+static void
+file_dialog_destroy (GtkWidget * widget, gpointer param)
+{
+  gtk_main_quit ();
+}
+
+/* Run the modal file selection dialog and return a new copy of the
+ * filename if the user pressed OK and NULL otherwise
+ */
+gchar *
+gpa_get_save_file_name (GtkWidget * parent, const gchar * title,
+			const gchar * directory)
+{
+  GPASaveFileNameDialog dialog;
+  GtkWidget * window = gtk_file_selection_new (title);
+
+  dialog.window = window;
+  dialog.filename = NULL;
+
+  gtk_signal_connect (GTK_OBJECT (window), "destroy",
+		      GTK_SIGNAL_FUNC (file_dialog_destroy), NULL);
+
+  gtk_signal_connect_object (GTK_OBJECT(GTK_FILE_SELECTION(window)->ok_button),
+			     "clicked", GTK_SIGNAL_FUNC (file_dialog_ok),
+			     (gpointer) &dialog);
+  gtk_signal_connect_object (
+		      GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
+		      "clicked", GTK_SIGNAL_FUNC (file_dialog_cancel),
+		      (gpointer) &dialog);
+
+  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+  gpa_window_show_centered (window, parent);
+
+  gtk_main ();
+
+  return dialog.filename;
+}
+
+gchar *
+gpa_get_load_file_name (GtkWidget * parent, const gchar * title,
+			const gchar * directory)
+{
+  return gpa_get_save_file_name (parent, title, directory);
+}
