@@ -1,5 +1,5 @@
 /* optionsmenu.c  -  The GNU Privacy Assistant
- *	Copyright (C) 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 2000, 2001 G-N-U GmbH.
  *
  * This file is part of GPA
  *
@@ -28,132 +28,48 @@
 #include "gtktools.h"
 #include "keysmenu.h"
 
-static GtkWidget *homeDirSelect;
-static GtkWidget *loadOptionsSelect;
-static GtkWidget *saveOptionsSelect;
 
-void
-gpa_homeDirSelect_ok (void)
-{
-  global_homeDirectory =
-    gtk_file_selection_get_filename (GTK_FILE_SELECTION (homeDirSelect));
-  gtk_widget_hide (homeDirSelect);
-}				/* gpa_homeDirSelect_ok */
-
-void
-gpa_homeDirSelect_init (gchar * title)
-{
-  homeDirSelect = gtk_file_selection_new (title);
-  gtk_signal_connect (GTK_OBJECT
-		      (GTK_FILE_SELECTION (homeDirSelect)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC (gpa_homeDirSelect_ok),
-		      NULL);
-  gtk_signal_connect_object (GTK_OBJECT
-			     (GTK_FILE_SELECTION (homeDirSelect)->
-			      cancel_button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_hide),
-			     (gpointer) homeDirSelect);
-  gtk_signal_connect (GTK_OBJECT (homeDirSelect), "delete_event",
-		      GTK_SIGNAL_FUNC (gtk_widget_hide), NULL);
-  gtk_signal_connect_object (GTK_OBJECT (homeDirSelect), "hide",
-			     GTK_SIGNAL_FUNC (gtk_widget_hide),
-			     (gpointer) global_windowTip);
-}				/* gpa_homeDirSelect_init */
-
-void
-gpa_loadOptionsSelect_ok (void)
-{
-  gpapa_load_options (gtk_file_selection_get_filename
-		      (GTK_FILE_SELECTION (loadOptionsSelect)),
-		      &global_keyserver, &global_defaultRecipients,
-		      &global_defaultKey, &global_homeDirectory, gpa_callback,
-		      global_windowMain);
-  gtk_widget_hide (loadOptionsSelect);
-}				/* gpa_loadOptionsSelect_ok */
-
-void
-gpa_loadOptionsSelect_init (gchar * title)
-{
-  loadOptionsSelect = gtk_file_selection_new (title);
-  gtk_signal_connect (GTK_OBJECT
-		      (GTK_FILE_SELECTION (loadOptionsSelect)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC (gpa_loadOptionsSelect_ok),
-		      NULL);
-  gtk_signal_connect_object (GTK_OBJECT
-			     (GTK_FILE_SELECTION (loadOptionsSelect)->
-			      cancel_button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_hide),
-			     (gpointer) loadOptionsSelect);
-  gtk_signal_connect (GTK_OBJECT (loadOptionsSelect), "delete_event",
-		      GTK_SIGNAL_FUNC (gtk_widget_hide), NULL);
-  gtk_signal_connect_object (GTK_OBJECT (loadOptionsSelect), "hide",
-			     GTK_SIGNAL_FUNC (gtk_widget_hide),
-			     (gpointer) global_windowTip);
-}				/* gpa_loadOptionsSelect_init */
-
-void
-gpa_saveOptionsSelect_ok (void)
-{
-  gpapa_save_options (gtk_file_selection_get_filename
-		      (GTK_FILE_SELECTION (saveOptionsSelect)),
-		      global_keyserver, global_defaultRecipients,
-		      global_defaultKey, global_homeDirectory, gpa_callback,
-		      global_windowMain);
-  gtk_widget_hide (saveOptionsSelect);
-}				/* gpa_saveOptionsSelect_ok */
-
-void
-gpa_saveOptionsSelect_init (gchar * title)
-{
-  saveOptionsSelect = gtk_file_selection_new (title);
-  gtk_signal_connect (GTK_OBJECT
-		      (GTK_FILE_SELECTION (saveOptionsSelect)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC (gpa_saveOptionsSelect_ok),
-		      NULL);
-  gtk_signal_connect_object (GTK_OBJECT
-			     (GTK_FILE_SELECTION (saveOptionsSelect)->
-			      cancel_button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_hide),
-			     (gpointer) saveOptionsSelect);
-  gtk_signal_connect (GTK_OBJECT (saveOptionsSelect), "delete_event",
-		      GTK_SIGNAL_FUNC (gtk_widget_hide), NULL);
-  gtk_signal_connect_object (GTK_OBJECT (saveOptionsSelect), "hide",
-			     GTK_SIGNAL_FUNC (gtk_widget_hide),
-			     (gpointer) global_windowTip);
-}				/* gpa_saveOptionsSelect_init */
-
-void
+static void
 options_keyserver_set (gpointer param)
 {
-/* var */
   gpointer *localParam;
   GtkWidget *comboServer;
   GpaWindowKeeper *keeperServer;
   gpointer paramDone[2];
-/* commands */
+  GtkWidget *entry;
+
   localParam = (gpointer *) param;
   comboServer = (GtkWidget *) localParam[0];
   keeperServer = (GpaWindowKeeper *) localParam[1];
-/*!!!
-  global_keyserver = gtk_entry_get_text (
-    GTK_ENTRY ( GTK_COMBO ( comboServer ) -> entry )
-  );
-!!!*/
+
+  entry = GTK_COMBO (comboServer)->entry;
+  /* FIXME: In this form, setting the default server is a memory leak */
+  global_keyserver = xstrdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+
   paramDone[0] = keeperServer;
   paramDone[1] = NULL;
   gpa_window_destroy (paramDone);
-}				/* options_keyserver_set */
+} /* options_keyserver_set */
 
-void
-options_keyserver (void)
+
+static void
+options_keyserver_destroy (GtkWidget * widget, gpointer param)
+{
+  gtk_main_quit ();
+}
+
+
+static void
+options_keyserver (gpointer param)
 {
-/* var */
   GpaWindowKeeper *keeper;
   GtkAccelGroup *accelGroup;
   GList *contentsServer = NULL;
   gpointer *paramClose;
   gpointer *paramSet;
-/* objects */
+  GtkWidget * main_window = param;
+  int i;
+
   GtkWidget *windowServer;
   GtkWidget *vboxServer;
   GtkWidget *hboxServer;
@@ -162,18 +78,21 @@ options_keyserver (void)
   GtkWidget *hButtonBoxServer;
   GtkWidget *buttonCancel;
   GtkWidget *buttonSet;
-/* commands */
+
   keeper = gpa_windowKeeper_new ();
   windowServer = gtk_window_new (GTK_WINDOW_DIALOG);
   gpa_windowKeeper_set_window (keeper, windowServer);
   gtk_window_set_title (GTK_WINDOW (windowServer), _("Set key server"));
   accelGroup = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (windowServer), accelGroup);
+  gtk_signal_connect (GTK_OBJECT (windowServer), "destroy",
+		      GTK_SIGNAL_FUNC (options_keyserver_destroy), NULL);
+
   vboxServer = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxServer), 5);
   hboxServer = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (hboxServer), 5);
-  labelServer = gtk_label_new (_(""));
+  labelServer = gtk_label_new ("");
   gtk_box_pack_start (GTK_BOX (hboxServer), labelServer, FALSE, FALSE, 0);
   comboServer = gtk_combo_new ();
   gpa_connect_by_accelerator (GTK_LABEL (labelServer),
@@ -181,13 +100,11 @@ options_keyserver (void)
 			      _("_Key server: "));
   gtk_combo_set_value_in_list (GTK_COMBO (comboServer), FALSE, FALSE);
 
-  {  int i;
-
-     for (i=0; opt.keyserver_names[i]; i++ ) {
-	contentsServer = g_list_append (contentsServer,
-					opt.keyserver_names[i]);
+  for (i=0; gpa_options.keyserver_names[i]; i++ )
+    {
+      contentsServer = g_list_append (contentsServer,
+				      gpa_options.keyserver_names[i]);
     }
-  }
 
   gtk_combo_set_popdown_strings (GTK_COMBO (comboServer), contentsServer);
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (comboServer)->entry),
@@ -203,9 +120,9 @@ options_keyserver (void)
   gpa_windowKeeper_add_param (keeper, paramClose);
   paramClose[0] = keeper;
   paramClose[1] = NULL;
-  buttonCancel = gpa_buttonCancel_new (accelGroup, "_Cancel", paramClose);
+  buttonCancel = gpa_buttonCancel_new (accelGroup, _("_Cancel"), paramClose);
   gtk_container_add (GTK_CONTAINER (hButtonBoxServer), buttonCancel);
-  buttonSet = gpa_button_new (accelGroup, "_Set");
+  buttonSet = gpa_button_new (accelGroup, _("_Set"));
   paramSet = (gpointer *) xmalloc (2 * sizeof (gpointer));
   gpa_windowKeeper_add_param (keeper, paramSet);
   paramSet[0] = comboServer;
@@ -217,11 +134,13 @@ options_keyserver (void)
   gtk_box_pack_start (GTK_BOX (vboxServer), hButtonBoxServer, FALSE, FALSE,
 		      0);
   gtk_container_add (GTK_CONTAINER (windowServer), vboxServer);
-  gpa_widget_show (windowServer, global_windowMain,
-		   _("options_keyserver.tip"));
-}				/* options_keyserver */
 
-void
+  gtk_window_set_modal (GTK_WINDOW (windowServer), TRUE);
+  gpa_window_show_centered (windowServer, main_window);
+  gtk_main ();
+} /* options_keyserver */
+
+static void
 options_recipients_fillDefault (gpointer data, gpointer userData)
 {
 /* var */
@@ -239,7 +158,7 @@ options_recipients_fillDefault (gpointer data, gpointer userData)
   gtk_clist_append (GTK_CLIST (clistDefault), contentsDefault);
 }				/* options_recipients_fillDefault */
 
-void
+static void
 options_recipients_set (gpointer param)
 {
 /* var */
@@ -264,14 +183,21 @@ options_recipients_set (gpointer param)
   gpa_recipientWindow_close (param);
 }				/* options_recipients_set */
 
-void
-options_recipients (void)
+
+static void
+options_recipients_destroy (GtkWidget * widget, gpointer param)
+{
+  gtk_main_quit ();
+}
+
+
+static void
+options_recipients (gpointer param)
 {
-/* var */
   GpaWindowKeeper *keeper;
   gint contentsKeyCount;
   GtkAccelGroup *accelGroup;
-  gchar *titlesAnyClist[] = { N_("Key owner"), N_("Key ID") };
+  gchar *titlesAnyClist[] = { _("Key owner"), _("Key ID") };
   gint i;
   GList **recipientsSelected = NULL;
   gchar *contentsAnyClist[2];
@@ -283,7 +209,8 @@ options_recipients (void)
   gpointer *paramAdd;
   gpointer *paramCancel;
   gpointer *paramSet;
-/* objects */
+  GtkWidget *parent = param;
+
   GtkWidget *windowRecipients;
   GtkWidget *vboxRecipients;
   GtkWidget *hboxRecipients;
@@ -302,7 +229,7 @@ options_recipients (void)
   GtkWidget *buttonAdd;
   GtkWidget *buttonCancel;
   GtkWidget *buttonSet;
-/* commands */
+
   contentsKeyCount =
     gpapa_get_public_key_count (gpa_callback, global_windowMain);
   if (!contentsKeyCount)
@@ -311,7 +238,8 @@ options_recipients (void)
 			("No public keys available to denote\nas default recipients."),
 global_windowMain);
       return;
-    }				/* if */
+    } /* if */
+
   keeper = gpa_windowKeeper_new ();
   windowRecipients = gtk_window_new (GTK_WINDOW_DIALOG);
   gpa_windowKeeper_set_window (keeper, windowRecipients);
@@ -319,12 +247,15 @@ global_windowMain);
 			_("Set default recipients"));
   accelGroup = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (windowRecipients), accelGroup);
+  gtk_signal_connect (GTK_OBJECT (windowRecipients), "destroy",
+		      GTK_SIGNAL_FUNC (options_recipients_destroy), NULL);
+  
   vboxRecipients = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxRecipients), 5);
   hboxRecipients = gtk_hbox_new (TRUE, 0);
   vboxDefault = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxDefault), 5);
-  labelDefault = gtk_label_new (_(""));
+  labelDefault = gtk_label_new ("");
   labelJfdDefault =
     gpa_widget_hjustified_new (labelDefault, GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (vboxDefault), labelJfdDefault, FALSE, FALSE,
@@ -356,7 +287,7 @@ global_windowMain);
   gtk_box_pack_start (GTK_BOX (hboxRecipients), vboxDefault, TRUE, TRUE, 0);
   vboxKeys = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxKeys), 5);
-  labelKeys = gtk_label_new (_(""));
+  labelKeys = gtk_label_new ("");
   labelJfdKeys = gpa_widget_hjustified_new (labelKeys, GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (vboxKeys), labelJfdKeys, FALSE, FALSE, 0);
   scrollerKeys = gtk_scrolled_window_new (NULL, NULL);
@@ -453,11 +384,13 @@ global_windowMain);
   gtk_box_pack_start (GTK_BOX (vboxRecipients), hButtonBoxRecipients, FALSE,
 		      FALSE, 0);
   gtk_container_add (GTK_CONTAINER (windowRecipients), vboxRecipients);
-  gpa_widget_show (windowRecipients, global_windowMain,
-		   _("options_recipients.tip"));
-}				/* options_recipients */
 
-void
+  gtk_window_set_modal (GTK_WINDOW (windowRecipients), TRUE);
+  gpa_window_show_centered (windowRecipients, parent);
+  gtk_main ();
+} /* options_recipients */
+
+static void
 options_key_select (GtkCList * clist, gint row, gint column,
 		    GdkEventButton * event, gpointer userData)
 {
@@ -468,7 +401,7 @@ options_key_select (GtkCList * clist, gint row, gint column,
   gtk_clist_get_text (clist, row, 1, keyID);
 }				/* options_key_select */
 
-void
+static void
 options_key_set (gpointer param)
 {
 /* var */
@@ -480,21 +413,26 @@ options_key_set (gpointer param)
   localParam = (gpointer *) param;
   keyID = (gchar **) localParam[0];
   keeperKey = (GpaWindowKeeper *) localParam[1];
-  if (global_defaultKey)
-    free (global_defaultKey);
-  global_defaultKey = xstrdup (*keyID);
+  gpa_set_default_key (xstrdup (*keyID));
   paramDone[0] = keeperKey;
   paramDone[1] = NULL;
   gpa_window_destroy (paramDone);
 }				/* options_key_set */
 
-void
-options_key (void)
+
+static void
+options_key_destroy (GtkWidget * widget, gpointer param)
+{
+  gtk_main_quit ();
+}
+
+
+static void
+options_key (gpointer param)
 {
-/* var */
   GpaWindowKeeper *keeper;
   GtkAccelGroup *accelGroup;
-  gchar *titlesKeys[] = { N_("User identity / role"), N_("Key ID") };
+  gchar *titlesKeys[] = { _("User identity / role"), _("Key ID") };
   gint contentsKeyCount;
   gchar **keyID;
   GpapaSecretKey *key;
@@ -502,7 +440,8 @@ options_key (void)
   gchar *contentsKeys[2];
   gpointer *paramSet;
   gpointer *paramClose;
-/* objects */
+  GtkWidget *parent = param;
+
   GtkWidget *windowKey;
   GtkWidget *vboxKey;
   GtkWidget *vboxKeys;
@@ -513,7 +452,7 @@ options_key (void)
   GtkWidget *hButtonBoxKey;
   GtkWidget *buttonCancel;
   GtkWidget *buttonSet;
-/* commands */
+
   contentsKeyCount =
     gpapa_get_secret_key_count (gpa_callback, global_windowMain);
   if (!contentsKeyCount)
@@ -523,16 +462,20 @@ options_key (void)
       return;
     }
   keeper = gpa_windowKeeper_new ();
+
   windowKey = gtk_window_new (GTK_WINDOW_DIALOG);
   gpa_windowKeeper_set_window (keeper, windowKey);
   gtk_window_set_title (GTK_WINDOW (windowKey), _("Set default key"));
   accelGroup = gtk_accel_group_new ();
   gtk_window_add_accel_group (GTK_WINDOW (windowKey), accelGroup);
+  gtk_signal_connect (GTK_OBJECT (windowKey), "destroy",
+		      (GtkSignalFunc)options_key_destroy, NULL);
+
   vboxKey = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxKey), 5);
   vboxKeys = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxKeys), 5);
-  labelKeys = gtk_label_new (_(""));
+  labelKeys = gtk_label_new ("");
   labelJfdKeys = gpa_widget_hjustified_new (labelKeys, GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (vboxKeys), labelJfdKeys, FALSE, FALSE, 0);
   scrollerKeys = gtk_scrolled_window_new (NULL, NULL);
@@ -560,12 +503,12 @@ options_key (void)
 				  global_windowMain);
       gtk_clist_prepend (GTK_CLIST (clistKeys), contentsKeys);
     }				/* while */
-  if (global_defaultKey)
+  if (gpa_default_key ())
     {
       i = 0;
       rows = GTK_CLIST (clistKeys)->rows;
       gtk_clist_get_text (GTK_CLIST (clistKeys), i, 1, keyID);
-      while (i < rows && strcmp (global_defaultKey, *keyID) != 0)
+      while (i < rows && strcmp (gpa_default_key (), *keyID) != 0)
 	{
 	  i++;
 	  if (i < rows)
@@ -601,68 +544,23 @@ options_key (void)
   gtk_container_add (GTK_CONTAINER (hButtonBoxKey), buttonSet);
   gtk_box_pack_start (GTK_BOX (vboxKey), hButtonBoxKey, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (windowKey), vboxKey);
-  gpa_widget_show (windowKey, global_windowMain, _("options_key.tip"));
-}				/* options_key */
+
+  gtk_window_set_modal (GTK_WINDOW (windowKey), TRUE);
+  gpa_window_show_centered (windowKey, parent);
+  gtk_main ();
+} /* options_key */
+
 
 void
-options_homedir (void)
+gpa_options_menu_add_to_factory (GtkItemFactory *factory, GtkWidget *window)
 {
-  gtk_widget_show (homeDirSelect);
-  gpa_windowTip_show (_("options_homedir.tip"));
-}				/* options_homedir */
+  GtkItemFactoryEntry menu[] = {
+    {_("/_Options"), NULL, NULL, 0, "<Branch>"},
+    {_("/Options/_Keyserver"), NULL, options_keyserver, 0, NULL},
+    {_("/Options/Default _Recipients"), NULL, options_recipients, 0, NULL},
+    {_("/Options/_Default Key"), NULL, options_key, 0, NULL},
+  };
 
-void
-options_tips (void)
-{
-/* var */
-  GpaWindowKeeper *keeper;
-  GtkAccelGroup *accelGroup;
-  gpointer *paramClose;
-/* objects */
-  GtkWidget *windowTips;
-  GtkWidget *hButtonBoxTips;
-  GtkWidget *toggleTips;
-  GtkWidget *buttonClose;
-/* commands */
-  keeper = gpa_windowKeeper_new ();
-  windowTips = gtk_window_new (GTK_WINDOW_DIALOG);
-  gpa_windowKeeper_set_window (keeper, windowTips);
-  gtk_window_set_title (GTK_WINDOW (windowTips), _("Show GPA Tips"));
-  accelGroup = gtk_accel_group_new ();
-  gtk_window_add_accel_group (GTK_WINDOW (windowTips), accelGroup);
-  hButtonBoxTips = gtk_hbutton_box_new ();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (hButtonBoxTips),
-			     GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hButtonBoxTips), 10);
-  gtk_container_set_border_width (GTK_CONTAINER (hButtonBoxTips), 10);
-  toggleTips = gpa_toggle_button_new (accelGroup, _("_Show GPA tips"));
-  if (global_noTips == FALSE)
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleTips), TRUE);
-  else
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleTips), FALSE);
-  gtk_signal_connect (GTK_OBJECT (toggleTips), "clicked",
-		      GTK_SIGNAL_FUNC (gpa_switch_tips), NULL);
-  gtk_container_add (GTK_CONTAINER (hButtonBoxTips), toggleTips);
-  paramClose = (gpointer *) xmalloc (2 * sizeof (gpointer));
-  gpa_windowKeeper_add_param (keeper, paramClose);
-  paramClose[0] = keeper;
-  paramClose[1] = NULL;
-  buttonClose = gpa_buttonCancel_new (accelGroup, _("_Close"), paramClose);
-  gtk_container_add (GTK_CONTAINER (hButtonBoxTips), buttonClose);
-  gtk_container_add (GTK_CONTAINER (windowTips), hButtonBoxTips);
-  gpa_widget_show (windowTips, global_windowMain, _("options_tips.tip"));
-}				/* options_tips */
-
-void
-options_load (void)
-{
-  gtk_widget_show (loadOptionsSelect);
-  gpa_windowTip_show (_("options_load.tip"));
-}				/* options_load */
-
-void
-options_save (void)
-{
-  gtk_widget_show (saveOptionsSelect);
-  gpa_windowTip_show (_("options_save.tip"));
-}				/* options_save */
+  gtk_item_factory_create_items (factory, sizeof (menu) / sizeof (menu[0]),
+				 menu, window);
+}
